# App::hopen::T::VSXML - Visual Studio toolset for MSBuild XML
package App::hopen::T::VSXML;
use Data::Hopen;
use strict; use warnings;
use Data::Hopen::Base;

our $VERSION = '0.000013'; # TRIAL

=head1 NAME

App::hopen::T::VSXML - hopen Visual Studio toolset for MSBuild projects

=head1 SYNOPSIS

This toolset creates XML records that can be used in an MSBuild project
under Microsoft Visual Studio.

=cut

1;
__END__
# vi: set fdm=marker: #
